@@ -20,6 +20,8 @@
 
 #include "Integrator6581.h"
 
+#include <cstdint>
+
 #ifdef SLOPE_FACTOR
 #  include <cmath>
 #  include "sidcxx11.h"
@@ -45,11 +47,11 @@ int Integrator6581::solve(int vi) const
     const unsigned int Vgdt_2 = Vgdt * Vgdt;
 
     // "Snake" current, scaled by (1/m)*2^13*m*2^16*m*2^16*2^-15 = m*2^30
-    const int n_I_snake = fmc->getNormalizedCurrentFactor(wlSnake) * (static_cast<int>(Vgst_2 - Vgdt_2) >> 15);
+    const int n_I_snake = fmc.getNormalizedCurrentFactor<13>(wlSnake) * (static_cast<int>(Vgst_2 - Vgdt_2) >> 15);
 
     // VCR gate voltage.       // Scaled by m*2^16
     // Vg = Vddt - sqrt(((Vddt - Vw)^2 + Vgdt^2)/2)
-    const int nVg = static_cast<int>(fmc->getVcr_nVg((nVddt_Vw_2 + (Vgdt_2 >> 1)) >> 16));
+    const int nVg = static_cast<int>(fmc.getVcr_nVg((nVddt_Vw_2 + (Vgdt_2 >> 1)) >> 16));
 #ifdef SLOPE_FACTOR
     const double nVp = static_cast<double>(nVg - nVt) / n; // Pinch-off voltage
     const int kVgt = static_cast<int>(nVp + 0.5) - nVmin;
@@ -58,14 +60,14 @@ int Integrator6581::solve(int vi) const
 #endif
 
     // VCR voltages for EKV model table lookup.
-    const int kVgt_Vs = (kVgt - vx) + (1 << 15);
-    assert((kVgt_Vs >= 0) && (kVgt_Vs < (1 << 16)));
-    const int kVgt_Vd = (kVgt - vi) + (1 << 15);
-    assert((kVgt_Vd >= 0) && (kVgt_Vd < (1 << 16)));
+    const int kVgt_Vs = (kVgt - vx) - INT16_MIN;
+    assert((kVgt_Vs >= 0) && (kVgt_Vs <= UINT16_MAX));
+    const int kVgt_Vd = (kVgt - vi) - INT16_MIN;
+    assert((kVgt_Vd >= 0) && (kVgt_Vd <= UINT16_MAX));
 
     // VCR current, scaled by m*2^15*2^15 = m*2^30
-    const unsigned int If = static_cast<unsigned int>(fmc->getVcr_n_Ids_term(kVgt_Vs)) << 15;
-    const unsigned int Ir = static_cast<unsigned int>(fmc->getVcr_n_Ids_term(kVgt_Vd)) << 15;
+    const unsigned int If = static_cast<unsigned int>(fmc.getVcr_n_Ids_term(kVgt_Vs)) << 15;
+    const unsigned int Ir = static_cast<unsigned int>(fmc.getVcr_n_Ids_term(kVgt_Vd)) << 15;
 #ifdef SLOPE_FACTOR
     const double iVcr = static_cast<double>(If - Ir);
     const int n_I_vcr = static_cast<int>(iVcr * n);
@@ -77,8 +79,8 @@ int Integrator6581::solve(int vi) const
     // estimate new slope factor based on gate voltage
     constexpr double gamma = 1.0;   // body effect factor
     constexpr double phi = 0.8;     // bulk Fermi potential
-    const double Vp = nVp / fmc->getN16();
-    n = 1. + (gamma / (2. * std::sqrt(Vp + phi + 4. * fmc->getUt())));
+    const double Vp = nVp / fmc.getN16();
+    n = 1. + (gamma / (2. * std::sqrt(Vp + phi + 4. * fmc.getUt())));
     assert((n > 1.2) && (n < 1.8));
 #endif
 
@@ -86,9 +88,9 @@ int Integrator6581::solve(int vi) const
     vc += n_I_snake + n_I_vcr;
 
     // vx = g(vc)
-    const int tmp = (vc >> 15) + (1 << 15);
-    assert(tmp < (1 << 16));
-    vx = fmc->getOpampRev(tmp);
+    const int tmp = (vc >> 15) - INT16_MIN;
+    assert(tmp <= UINT16_MAX);
+    vx = fmc.getOpampRev(tmp);
 
     // Return vo.
     return vx - (vc >> 14);
